@@ -1,6 +1,6 @@
 /* ACVP Proxy application
  *
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -50,6 +50,7 @@
 struct opt_data {
 	struct acvp_search_ctx search;
 	struct acvp_opts_ctx acvp_ctx_options;
+	struct acvp_rename_ctx rename_ctx;
 	char *specific_modversion;
 
 	char *seed_base64;
@@ -69,11 +70,13 @@ struct opt_data {
 	size_t cipher_options_algo_idx;
 	bool cipher_list;
 
+	bool rename;
 	bool request;
 	bool publish;
 	bool list_available_ids;
 	bool list_pending_request_ids;
 	bool list_verdicts;
+	bool list_certificates;
 	bool dump_register;
 	bool request_sample;
 	bool official_testing;
@@ -160,7 +163,7 @@ static int read_complete(int fd, char *buf, uint32_t buflen)
 	do {
 		ret = read(fd, buf, buflen);
 		if (0 < ret) {
-			buflen -= ret;
+			buflen -= (uint32_t)ret;
 			buf += ret;
 		}
 	} while ((0 < ret || EINTR == errno) && buflen);
@@ -251,8 +254,8 @@ static int load_config(struct opt_data *opts)
 		goto out;
 	}
 
-	opts->seed_base64_len = statbuf.st_size;
-	opts->seed_base64 = malloc(statbuf.st_size);
+	opts->seed_base64_len = (uint32_t)statbuf.st_size;
+	opts->seed_base64 = malloc((size_t)statbuf.st_size);
 	if (!opts->seed_base64) {
 		ret = -ENOMEM;
 		goto out;
@@ -303,15 +306,17 @@ static void usage(void)
 	fprintf(stderr, "\t\t\t\t\tto process\n");
 	fprintf(stderr, "\t-p --processor <NAME>\t\tDefinition search criteria: Name of \n");
 	fprintf(stderr, "\t\t\t\t\tprocessor executing crypto module\n");
-	fprintf(stderr, "\t-f --fuzzy\t\t\tPerform a fuzzy search (perform a\n");
-	fprintf(stderr, "\t\t\t\t\tsubstring search)\n");
+	fprintf(stderr, "\t-f --fuzzy\t\t\tPerform a fuzzy search for all search \n");
+	fprintf(stderr, "\t\t\t\t\tcriteria (perform a substring search)\n");
 	fprintf(stderr, "\n\tNote: The search critera allow narrowing the processed module\n");
 	fprintf(stderr, "\tdefinitions found with the list operation. The less search criteria are\n");
 	fprintf(stderr, "\tprovided, the broader the scope is. If no search critera are provided,\n");
-
 	fprintf(stderr, "\tall module definitions are in scope. For example, using --request with\n");
 	fprintf(stderr, "\tno search criteria implies that test vectors for all module\n");
 	fprintf(stderr, "\timplementations known to the library are requested.\n\n");
+	fprintf(stderr, "\tNote 2: Prepending the search strings with \"f:\" requests a fuzzy search\n");
+	fprintf(stderr, "\tfor only that particular search criteria.\n\n");
+
 	fprintf(stderr, "\t-c --config\t\t\tConfiguration file\n");
 	fprintf(stderr, "\t-l --list\t\t\tList supported crypto modules\n");
 	fprintf(stderr, "\t-u --unregistered\t\tList unregistered crypto definitions\n");
@@ -346,7 +351,14 @@ static void usage(void)
 	fprintf(stderr, "\t\t\t\t\tcurrent results on file to update the\n");
 	fprintf(stderr, "\t\t\t\t\tresults on the ACVP server\n\n");
 
+	fprintf(stderr, "\t   --delete-test\t\tDelete all vsIds in scope which\n");
+	fprintf(stderr, "\t\t\t\t\tare part of the registration - this\n");
+	fprintf(stderr, "\t\t\t\t\toption is only applicable during\n");
+	fprintf(stderr, "\t\t\t\t\tsubmitting responses to the ACVP server\n\n");
+
 	fprintf(stderr, "\tUpdate ACVP database with content from JSON configuration files:\n");
+	fprintf(stderr, "\t   --nopublish-prereqs\t\tRemove prerequisites from publication\n");
+	fprintf(stderr, "\t\t\t\t\trequest\n");
 	fprintf(stderr, "\t   --register-definition\tRegister pending definitions with ACVP\n");
 	fprintf(stderr, "\t   --delete-definition <TYPE>\tDelete definition at ACVP server\n");
 	fprintf(stderr, "\t   --update-definition <TYPE>\tUpdate definition at ACVP server\n");
@@ -362,6 +374,7 @@ static void usage(void)
 	fprintf(stderr, "\t   --list-request-ids\t\tList all pending request IDs\n");
 	fprintf(stderr, "\t   --list-available-ids\t\tList all available IDs\n");
 	fprintf(stderr, "\t   --list-verdicts\t\tList all verdicts\n\n");
+	fprintf(stderr, "\t   --list-certificates\t\tList all certificates\n\n");
 
 	fprintf(stderr, "\tGathering cipher definitions from ACVP server:\n");
 	fprintf(stderr, "\t   --cipher-list\t\tList all ciphers supported by ACVP\n");
@@ -371,6 +384,11 @@ static void usage(void)
 	fprintf(stderr, "\t   --cipher-algo <ALGO>\t\tGet cipher options particular cipher\n\n");
 
 	fprintf(stderr, "\tAuxiliary options:\n");
+	fprintf(stderr, "\t   --proxy-extension <SO-FILE>\tShared library of ACVP Proxy extension\n");
+	fprintf(stderr, "\t\t\t\t\tZero or more extensions can be provided.\n");
+	fprintf(stderr, "\t   --rename-version <NEW>\tRename version of definition\n");
+	fprintf(stderr, "\t   --rename-name <NEW_NAME>\tRename name of definition\n");
+	fprintf(stderr, "\t   --rename-oename <NEW_NAME>\tRename OE name of definition\n");
 	fprintf(stderr, "\t-v --verbose\t\t\tVerbose logging, multiple options\n");
 	fprintf(stderr, "\t\t\t\t\tincrease verbosity\n");
 	fprintf(stderr, "\t\t\t\t\tNote: In debug mode (3 or more -v),\n");
@@ -379,6 +397,7 @@ static void usage(void)
 	fprintf(stderr, "\t\t\t\t\t(-1 lists all logging classes)\n");
 	fprintf(stderr, "\t-q --quiet\t\t\tNo output - quiet operation\n");
 	fprintf(stderr, "\t   --version\t\t\tVersion of ACVP proxy\n");
+	fprintf(stderr, "\t   --version-numeric\t\tNumeric version of ACVP proxy\n");
 	fprintf(stderr, "\t-h --help\t\t\tPrint this help information\n");
 }
 
@@ -416,7 +435,7 @@ static void free_opts(struct opt_data *opts)
 	json_object_put(opts->config);
 }
 
-static int duplicate_string(char **dst, char *src)
+static int duplicate_string(char **dst, const char *src)
 {
 	if (*dst)
 		free(*dst);
@@ -440,7 +459,7 @@ static bool ask_yes(const char *question)
 	fprintf_red(stdout, "%s (Y/N)? ", question);
 
 	while (1) {
-		answer = fgetc(stdin);
+		answer = (unsigned char)fgetc(stdin);
 
 		switch (answer) {
 		case 'y':
@@ -482,9 +501,45 @@ static int convert_update_delete_type(const char *string, unsigned int *option)
 	return 0;
 }
 
+static int parse_fuzzy_flag(bool *fuzzy_search_flag, char **dst,
+			    const char *src)
+{
+	int ret;
+	char *fuzzing_request;
+
+	if (!src) {
+		logger(LOGGER_ERR, LOGGER_C_ANY,
+		       "Empty string for fuzzy search provided\n");
+		return -EINVAL;
+	}
+
+	fuzzing_request = strstr(src, "f:");
+
+	/*
+	 * Only honor the fuzzing request string if placed at the beginning
+	 * of the string.
+	 */
+	if (fuzzing_request && fuzzing_request == src) {
+		*fuzzy_search_flag = true;
+		src += 2;
+	}
+
+	/*
+	 * In case no fuzzy search flag is provided, we do NOT set the
+	 * *fuzzy_search_flag to false, because the caller may have provided
+	 * the -f command line option.
+	 */
+
+	CKINT(duplicate_string(dst, src));
+
+out:
+	return ret;
+}
+
 static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 {
 	struct acvp_search_ctx *search = &opts->search;
+	struct acvp_rename_ctx *rename = &opts->rename_ctx;
 	int c = 0, ret;
 	char version[200] = { 0 };
 	unsigned long val = 0;
@@ -500,6 +555,7 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			{"logger-class",	required_argument,	0, 0},
 			{"quiet",		no_argument,		0, 'q'},
 			{"version",		no_argument,		0, 0},
+			{"version-numeric",	no_argument,		0, 0},
 			{"help",		no_argument,		0, 'h'},
 
 			{"module",		required_argument,	0, 'm'},
@@ -527,18 +583,26 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			{"definition-basedir",	required_argument,	0, 0},
 
 			{"resubmit-result",	no_argument,		0, 0},
+			{"delete-test",		no_argument,		0, 0},
 			{"register-definition",	no_argument,		0, 0},
 			{"delete-definition",	required_argument,	0, 0},
 			{"update-definition",	required_argument,	0, 0},
+			{"nopublish-prereqs",	required_argument,	0, 0},
 
 			{"list-request-ids",	no_argument,		0, 0},
 			{"list-available-ids",	no_argument,		0, 0},
 			{"list-verdicts",	no_argument,		0, 0},
+			{"list-certificates",	no_argument,		0, 0},
 
 			{"cipher-options",	required_argument,	0, 0},
 			{"cipher-algo",		required_argument,	0, 0},
 			{"cipher-list",		no_argument,		0, 0},
 
+			{"proxy-extension",	required_argument,	0, 0},
+			{"rename-version",	required_argument,	0, 0},
+			{"rename-name",		required_argument,	0, 0},
+			{"rename-oename",	required_argument,	0, 0},
+
 			{0, 0, 0, 0}
 		};
 		c = getopt_long(argc, argv, "m:n:e:r:p:fluc:d:ob:s:vqh", options,
@@ -550,9 +614,12 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			switch (opt_index) {
 			case 0:
 				logger_inc_verbosity();
+				if (logger_get_verbosity(LOGGER_C_ANY) >=
+							LOGGER_DEBUG)
+					opts->acvp_ctx_options.threading_disabled = true;
 				break;
 			case 1:
-				lval = strtoul(optarg, NULL, 10);
+				lval = strtol(optarg, NULL, 10);
 				if (lval == LONG_MAX) {
 					logger(LOGGER_ERR, LOGGER_C_ANY,
 					       "undefined logging class value\n");
@@ -582,46 +649,61 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 				goto out;
 				break;
 			case 4:
+				fprintf(stderr, "%u\n",
+					acvp_versionstring_numeric());
+				ret = 0;
+				goto out;
+				break;
+			case 5:
 				usage();
 				ret = 0;
 				goto out;
 				break;
 
-			case 5:
-				CKINT(duplicate_string(&search->modulename,
-						       optarg));
-				break;
 			case 6:
-				CKINT(duplicate_string(&search->vendorname,
-						       optarg));
+				CKINT(parse_fuzzy_flag(
+					&search->modulename_fuzzy_search,
+					&search->modulename, optarg));
 				break;
 			case 7:
-				CKINT(duplicate_string(&search->execenv,
-						       optarg));
+				CKINT(parse_fuzzy_flag(
+					&search->vendorname_fuzzy_search,
+					&search->vendorname, optarg));
 				break;
 			case 8:
-				CKINT(duplicate_string(&search->moduleversion,
-						       optarg));
+				CKINT(parse_fuzzy_flag(
+					&search->execenv_fuzzy_search,
+					&search->execenv, optarg));
 				break;
 			case 9:
-				CKINT(duplicate_string(&search->processor,
-						       optarg));
+				CKINT(parse_fuzzy_flag(
+					&search->moduleversion_fuzzy_search,
+					&search->moduleversion, optarg));
 				break;
 			case 10:
-				search->fuzzy_name_search = true;
+				CKINT(parse_fuzzy_flag(
+					&search->processor_fuzzy_search,
+					&search->processor, optarg));
+				break;
+			case 11:
+				search->modulename_fuzzy_search = true;
+				search->moduleversion_fuzzy_search = true;
+				search->vendorname_fuzzy_search = true;
+				search->execenv_fuzzy_search = true;
+				search->processor_fuzzy_search = true;
 				break;
 
-			case 11:
+			case 12:
 				dolist = 1;
 				break;
-			case 12:
+			case 13:
 				listunregistered = 1;
 				break;
-			case 13:
+			case 14:
 				CKINT(duplicate_string(&opts->specific_modversion,
 						       optarg));
 				break;
-			case 14:
+			case 15:
 				if (search->nr_submit_vsid >= MAX_SUBMIT_ID) {
 					logger(LOGGER_ERR, LOGGER_C_ANY,
 					       "%u --submit options are allowed at maximum\n",
@@ -637,9 +719,9 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 					ret = -EINVAL;
 					goto out;
 				}
-				search->submit_vsid[search->nr_submit_vsid++] = val;
+				search->submit_vsid[search->nr_submit_vsid++] = (unsigned int)val;
 				break;
-			case 15:
+			case 16:
 				if (search->nr_submit_testid >= MAX_SUBMIT_ID) {
 					logger(LOGGER_ERR, LOGGER_C_ANY,
 					       "%u --session options are allowed at maximum\n",
@@ -655,76 +737,89 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 					ret = -EINVAL;
 					goto out;
 				}
-				search->submit_testid[search->nr_submit_testid++] = val;
-				break;
-			case 16:
-				opts->request = true;
+				search->submit_testid[search->nr_submit_testid++] = (unsigned int)val;
 				break;
 			case 17:
+				opts->request = true;
+				break;
+			case 18:
 				opts->publish = true;
 				break;
 
-			case 18:
+			case 19:
 				opts->dump_register = true;
 				break;
-			case 19:
+			case 20:
 				opts->request_sample = true;
 				break;
-			case 20:
+			case 21:
 				CKINT(duplicate_string(&opts->configfile,
 						       optarg));
 				break;
-			case 21:
+			case 22:
 				CKINT(acvp_def_config(optarg));
 				modconf_loaded = 1;
 				break;
-			case 22:
+			case 23:
 				opts->official_testing = true;
 				break;
-			case 23:
+			case 24:
 				CKINT(duplicate_string(&opts->basedir, optarg));
 				break;
-			case 24:
+			case 25:
 				CKINT(duplicate_string(&opts->secure_basedir,
 						       optarg));
 				break;
-			case 25:
+			case 26:
 				CKINT(duplicate_string(&opts->definition_basedir,
 						       optarg));
 				break;
 
-			case 26:
+			case 27:
 				opts->acvp_ctx_options.resubmit_result = true;
 				break;
-			case 27:
+			case 28:
+				opts->acvp_ctx_options.delete_vsid = true;
+				break;
+			case 29:
 				opts->acvp_ctx_options.register_new_module = true;
 				opts->acvp_ctx_options.register_new_vendor = true;
 				opts->acvp_ctx_options.register_new_oe = true;
 				break;
-			case 28:
+			case 30:
 				CKINT(convert_update_delete_type(optarg,
 				  &opts->acvp_ctx_options.delete_db_entry));
 				break;
-			case 29:
+			case 31:
 				CKINT(convert_update_delete_type(optarg,
 				  &opts->acvp_ctx_options.update_db_entry));
 				break;
-
-			case 30:
-				opts->list_pending_request_ids = true;
-				break;
-			case 31:
-				opts->list_available_ids = true;
-				break;
 			case 32:
-				opts->list_verdicts = true;
+				opts->acvp_ctx_options.no_publish_prereqs = true;
 				break;
 
 			case 33:
+				opts->list_pending_request_ids = true;
+				opts->acvp_ctx_options.threading_disabled = true;
+				break;
+			case 34:
+				opts->list_available_ids = true;
+				opts->acvp_ctx_options.threading_disabled = true;
+				break;
+			case 35:
+				opts->list_verdicts = true;
+				opts->acvp_ctx_options.threading_disabled = true;
+				break;
+			case 36:
+				opts->list_certificates = true;
+				opts->acvp_ctx_options.threading_disabled = true;
+				break;
+
+			case 37:
 				CKINT(duplicate_string(&opts->cipher_options_file,
 						       optarg));
 				break;
-			case 34:
+			case 38:
 				CKINT(duplicate_string(&opts->cipher_options_algo[opts->cipher_options_algo_idx],
 						       optarg));
 				opts->cipher_options_algo_idx++;
@@ -734,10 +829,29 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 					goto out;
 				}
 				break;
-			case 35:
+			case 39:
 				opts->cipher_list = true;
 				break;
 
+			case 40:
+				CKINT(acvp_load_extension(optarg));
+				break;
+			case 41:
+				rename->moduleversion_new = optarg;
+				opts->acvp_ctx_options.threading_disabled = true;
+				opts->rename = true;
+				break;
+			case 42:
+				rename->modulename_new = optarg;
+				opts->acvp_ctx_options.threading_disabled = true;
+				opts->rename = true;
+				break;
+			case 43:
+				rename->oe_env_name_new = optarg;
+				opts->acvp_ctx_options.threading_disabled = true;
+				opts->rename = true;
+				break;
+
 			default:
 				usage();
 				ret = -EINVAL;
@@ -747,22 +861,32 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			break;
 
 		case 'm':
-			CKINT(duplicate_string(&search->modulename, optarg));
+			CKINT(parse_fuzzy_flag(&search->modulename_fuzzy_search,
+					       &search->modulename, optarg));
 			break;
 		case 'n':
-			CKINT(duplicate_string(&search->vendorname, optarg));
+			CKINT(parse_fuzzy_flag(&search->vendorname_fuzzy_search,
+					       &search->vendorname, optarg));
 			break;
 		case 'e':
-			CKINT(duplicate_string(&search->execenv, optarg));
+			CKINT(parse_fuzzy_flag(&search->execenv_fuzzy_search,
+					       &search->execenv, optarg));
 			break;
 		case 'r':
-			CKINT(duplicate_string(&search->moduleversion, optarg));
+			CKINT(parse_fuzzy_flag(
+					&search->moduleversion_fuzzy_search,
+					&search->moduleversion, optarg));
 			break;
 		case 'p':
-			CKINT(duplicate_string(&search->processor, optarg));
+			CKINT(parse_fuzzy_flag(&search->processor_fuzzy_search,
+					       &search->processor, optarg));
 			break;
 		case 'f':
-			search->fuzzy_name_search = true;
+			search->modulename_fuzzy_search = true;
+			search->moduleversion_fuzzy_search = true;
+			search->vendorname_fuzzy_search = true;
+			search->execenv_fuzzy_search = true;
+			search->processor_fuzzy_search = true;
 			break;
 
 		case 'l':
@@ -790,6 +914,9 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 
 		case 'v':
 			logger_inc_verbosity();
+			if (logger_get_verbosity(LOGGER_C_ANY) >=
+						 LOGGER_DEBUG)
+				opts->acvp_ctx_options.threading_disabled = true;
 			break;
 		case 'q':
 			logger_set_verbosity(LOGGER_NONE);
@@ -1128,6 +1255,20 @@ out:
 	return ret;
 }
 
+static int list_certificates(struct opt_data *opts)
+{
+	struct acvp_ctx *ctx = NULL;
+	int ret;
+
+	CKINT(initialize_ctx(&ctx, opts));
+
+	CKINT(acvp_list_certificates(ctx));
+
+out:
+	acvp_ctx_release(ctx);
+	return ret;
+}
+
 static int do_fetch_cipher_options(struct opt_data *opts)
 {
 	struct acvp_ctx *ctx = NULL;
@@ -1144,6 +1285,22 @@ out:
 	return ret;
 }
 
+static int do_rename(struct opt_data *opts)
+{
+	struct acvp_ctx *ctx = NULL;
+	int ret;
+
+	CKINT(initialize_ctx(&ctx, opts));
+
+	ctx->rename = &opts->rename_ctx;
+
+	CKINT(acvp_rename_module(ctx));
+
+out:
+	acvp_ctx_release(ctx);
+	return ret;
+}
+
 int main(int argc, char *argv[])
 {
 	struct opt_data opts;
@@ -1160,8 +1317,11 @@ int main(int argc, char *argv[])
 
 	CKINT(set_totp_seed(&opts));
 
-	if (opts.cipher_list || opts.cipher_options_file) {
+	if (opts.cipher_list || opts.cipher_options_algo_idx ||
+	    opts.cipher_options_file) {
 		CKINT(do_fetch_cipher_options(&opts));
+	} else if (opts.rename) {
+		CKINT(do_rename(&opts));
 	} else if (opts.request) {
 		CKINT(do_register(&opts));
 	} else if (opts.publish) {
@@ -1171,6 +1331,8 @@ int main(int argc, char *argv[])
 		CKINT(list_ids(&opts));
 	} else if (opts.list_verdicts) {
 		CKINT(list_verdicts(&opts));
+	} else if (opts.list_certificates) {
+		CKINT(list_certificates(&opts));
 	} else {
 		CKINT(do_submit(&opts));
 	}
