@@ -1,6 +1,6 @@
 /* ACVP proxy protocol handler
  *
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -18,10 +18,13 @@
  * DAMAGE.
  */
 
+#include <dlfcn.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/mman.h>
 #include <unistd.h>
 
@@ -86,6 +89,7 @@ static void acvp_release_datastore(struct acvp_datastore_ctx *datastore)
 	ACVP_PTR_FREE_NULL(datastore->jwttokenfile);
 	ACVP_PTR_FREE_NULL(datastore->messagesizeconstraint);
 	ACVP_PTR_FREE_NULL(datastore->testsession_certificate_id);
+	ACVP_PTR_FREE_NULL(datastore->testsession_certificate_info);
 	ACVP_PTR_FREE_NULL(datastore->verdictfile);
 	ACVP_PTR_FREE_NULL(datastore->processedfile);
 	ACVP_PTR_FREE_NULL(datastore->srcserver);
@@ -234,6 +238,25 @@ bool acvp_req_is_production(void)
 	return acvp_production;
 }
 
+static int acvp_check_file_presence(const char *file, const char *loginfo)
+{
+	struct stat statbuf;
+	int ret;
+
+	if (!file)
+		return 0;
+
+	if (stat(file, &statbuf)) {
+		ret = -errno;
+		logger(LOGGER_ERR, LOGGER_C_ANY,
+		       "Error accessing %s file %s (%s)\n", loginfo, file,
+		       strerror(errno));
+		return ret;
+	}
+
+	return 0;
+}
+
 /*****************************************************************************
  * API calls
  *****************************************************************************/
@@ -266,16 +289,24 @@ int acvp_set_net(const char *server_name, unsigned int port, const char *ca,
 
 	if (ca) {
 		CKINT(acvp_duplicate(&net->certs_ca_file, ca));
+		CKINT(acvp_check_file_presence(net->certs_ca_file, "CA"));
+		CKINT(acvp_cert_type(net->certs_ca_file,
+				     net->certs_ca_file_type,
+				     sizeof(net->certs_ca_file_type)));
 	} else {
 		net->certs_ca_file = NULL;
 	}
 	CKINT(acvp_duplicate(&net->certs_clnt_file, client_cert));
+	CKINT(acvp_check_file_presence(net->certs_clnt_file,
+				       "client certificate"));
 	CKINT(acvp_cert_type(net->certs_clnt_file,
 			     net->certs_clnt_file_type,
 			     sizeof(net->certs_clnt_file_type)));
 
 	if (client_key) {
 		CKINT(acvp_duplicate(&net->certs_clnt_key_file, client_key));
+		CKINT(acvp_check_file_presence(net->certs_clnt_key_file,
+					       "client key"));
 		CKINT(acvp_cert_type(net->certs_clnt_key_file,
 				     net->certs_clnt_key_file_type,
 				     sizeof(net->certs_clnt_key_file_type)));
@@ -354,7 +385,16 @@ int acvp_set_module(struct acvp_ctx *ctx,
 	CKINT(acvp_duplicate(&ctx_search->execenv, caller_search->execenv));
 	CKINT(acvp_duplicate(&ctx_search->processor, caller_search->processor));
 
-	ctx_search->fuzzy_name_search = caller_search->fuzzy_name_search;
+	ctx_search->modulename_fuzzy_search =
+				caller_search->modulename_fuzzy_search;
+	ctx_search->moduleversion_fuzzy_search =
+				caller_search->moduleversion_fuzzy_search;
+	ctx_search->vendorname_fuzzy_search =
+				caller_search->vendorname_fuzzy_search;
+	ctx_search->execenv_fuzzy_search =
+				caller_search->execenv_fuzzy_search;
+	ctx_search->processor_fuzzy_search =
+				caller_search->processor_fuzzy_search;
 
 	for (i = 0; i < caller_search->nr_submit_testid; i++)
 		ctx_search->submit_testid[i] = caller_search->submit_testid[i];
@@ -421,6 +461,18 @@ int acvp_versionstring_short(char *buf, size_t buflen)
 			MAJVERSION, MINVERSION, PATCHLEVEL);
 }
 
+DSO_PUBLIC
+uint32_t acvp_versionstring_numeric(void)
+{
+	uint32_t version = 0;
+
+	version =  MAJVERSION * 1000000;
+	version += MINVERSION * 10000;
+	version += PATCHLEVEL * 100;
+
+	return version;
+}
+
 #ifdef CRYPTOVERSION
 #define _CRYPTOVERSION CRYPTOVERSION
 #else
@@ -541,6 +593,8 @@ int acvp_ctx_init(struct acvp_ctx **ctx,
 			     ACVP_DS_MESSAGESIZECONSTRAINT));
 	CKINT(acvp_duplicate(&datastore->testsession_certificate_id,
 			     ACVP_DS_TESTSESSIONCERTIFICATEID));
+	CKINT(acvp_duplicate(&datastore->testsession_certificate_info,
+			     ACVP_DS_TESTSESSIONCERTIFICATEINFO));
 	CKINT(acvp_duplicate(&datastore->verdictfile, ACVP_DS_VERDICT));
 	CKINT(acvp_duplicate(&datastore->processedfile, ACVP_DS_PROCESSED));
 	CKINT(acvp_duplicate(&datastore->srcserver, ACVP_DS_SRCSERVER));
@@ -601,3 +655,25 @@ int acvp_init(const uint8_t *seed, uint32_t seed_len, time_t last_gen,
 out:
 	return ret;
 }
+
+DSO_PUBLIC
+int acvp_load_extension(const char *path)
+{
+	struct acvp_extension *extension = NULL;
+	int ret = 0;
+	void *library_handle = NULL;
+
+	library_handle = dlopen(path, RTLD_NOW);
+	CKNULL_LOG(library_handle, -EFAULT, "Error loading library: %s\n",
+		   dlerror());
+
+	dlerror();
+	extension = (struct acvp_extension *)dlsym(library_handle,
+						   "acvp_extension");
+	CKNULL_LOG(extension, -EFAULT,
+		   "Error finding symbol acvp_extension: %s\n", dlerror());
+	acvp_register_algo_map(extension->curr_map, extension->nrmaps);
+
+out:
+	return ret;
+}
