@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -56,9 +56,10 @@ extern "C"
 #define NIST_VAL_OP_ALGORITHMS		"algorithms"
 #define NIST_VAL_OP_LARGE		"large"
 #define NIST_VAL_OP_REQUESTS		"requests"
+#define NIST_VAL_OP_VALIDATIONS		"validations"
 
 /* acvp_protocol.txt: section 11.1 */
-#define ACVP_JWT_TOKEN_MAX      	1024
+#define ACVP_JWT_TOKEN_MAX      	16384
 /* lifetime of token in seconds - we subtract some grace time */
 #define ACVP_JWT_TOKEN_LIFETIME		(1800 - 300)
 
@@ -74,11 +75,18 @@ struct acvp_auth_ctx {
 	uint32_t max_reg_msg_size;
 
 	/*
-	 * Certificate ID (or request for it) of test session. If 0, no
-	 * test session certificate ID was applied for yet.
+	 * Certificate request ID of test session. If 0, no test session
+	 * certificate request ID was applied for yet.
 	 */
 	uint32_t testsession_certificate_id;
 
+	/*
+	 * Certificate number of test session. If NULL, no test session
+	 * certificate number was awarded yet. Note, if this value is non-NULL
+	 * the publication of the test session is complete.
+	 */
+	char *testsession_certificate_number;
+
 	/*
 	 * As auth ctx is shared between multiple vsIds belonging to one testId,
 	 * we must lock if we want to change it.
@@ -107,21 +115,33 @@ struct acvp_modinfo_ctx {
 /**
  * @brief Search parameters that can be set by the caller. If an entry is NULL
  *	  it is not used as a search criteria.
- * @param modulename Module name as reported by the list operation
- * @param moduleversion Version string as reported by the list operation
- * @param vendorname Vendor name as reported by the list operation
- * @param execenv Execution environment as reported by the list operation
- * @param processor Processor name as reported by the list operation
- * @param fuzzy_name_search Use the given search strings in a fuzzy search
- *			    (i.e. use strstr to search the name and not
- *			     strncmp)
+ * @var modulename Module name as reported by the list operation
+ * @var moduleversion Version string as reported by the list operation
+ * @var vendorname Vendor name as reported by the list operation
+ * @var execenv Execution environment as reported by the list operation
+ * @var processor Processor name as reported by the list operation
+ * @var modulename_fuzzy_search Use the given search strings in a fuzzy search
+ *			    	(i.e. use strstr to search the name and not
+ *			    	strncmp) for modulename
+ * @var moduleversion_fuzzy_search Use the given search strings in a fuzzy
+ *				   search (i.e. use strstr to search the name
+ *				   and not strncmp) for moduleversion
+ * @var vendorname_fuzzy_search Use the given search strings in a fuzzy search
+ *			    	(i.e. use strstr to search the name and not
+ *			    	strncmp) for vendorname
+ * @var execenv_fuzzy_search Use the given search strings in a fuzzy search
+ *			     (i.e. use strstr to search the name and not
+ *			     strncmp) for execenv
+ * @var processor_fuzzy_search Use the given search strings in a fuzzy search
+ *			       (i.e. use strstr to search the name and not
+ *			       strncmp) for processor
  *
- * @param submit_vsids Array of vsIds to be processed (if empty, all vsIDs are
- *		       in scope).
- * @param nr_submit_vsids Number of vsIDs that are to be searched.
- * @param submit_testids Array of testIds to be processed (if empty, all
- *			 testIDs are in scope).
- * @param nr_submit_testids Number of testIDs that are to be searched.
+ * @var submit_vsids Array of vsIds to be processed (if empty, all vsIDs are
+ *		     in scope).
+ * @var nr_submit_vsids Number of vsIDs that are to be searched.
+ * @var submit_testids Array of testIds to be processed (if empty, all
+ *		       testIDs are in scope).
+ * @var nr_submit_testids Number of testIDs that are to be searched.
  */
 struct acvp_search_ctx {
 	char *modulename;
@@ -129,7 +149,11 @@ struct acvp_search_ctx {
 	char *vendorname;
 	char *execenv;
 	char *processor;
-	bool fuzzy_name_search;
+	bool modulename_fuzzy_search;
+	bool moduleversion_fuzzy_search;
+	bool vendorname_fuzzy_search;
+	bool execenv_fuzzy_search;
+	bool processor_fuzzy_search;
 
 	unsigned int submit_vsid[MAX_SUBMIT_ID];
 	unsigned int nr_submit_vsid;
@@ -149,6 +173,7 @@ struct acvp_datastore_ctx {
 	char *jwttokenfile;
 	char *messagesizeconstraint;
 	char *testsession_certificate_id;
+	char *testsession_certificate_info;
 	char *verdictfile;
 	char *processedfile;
 	char *srcserver;
@@ -233,6 +258,19 @@ struct acvp_opts_ctx {
 	 */
 	bool register_new_module;
 
+	/* Disable threading */
+	bool threading_disabled;
+
+	/*
+	 * Generate array with prerequisites during publication phase.
+	 */
+	bool no_publish_prereqs;
+
+	/*
+	 * Delete the VSID the operation applies to
+	 */
+	bool delete_vsid;
+
 	/*
 	 * Delete an entry in the ACVP database. The ID is taken from the
 	 * module's JSON configuration file.
@@ -253,11 +291,25 @@ struct acvp_opts_ctx {
 	unsigned int update_db_entry;
 };
 
+/**
+ * @brief Specification of information to be renamed
+ *
+ * @var moduleversion_new new module version string
+ * @var modulename_new new module name
+ * @var oe_env_name_new new OE name
+ */
+struct acvp_rename_ctx {
+	const char *moduleversion_new;
+	const char *modulename_new;
+	const char *oe_env_name_new;
+};
+
 struct acvp_ctx {
 	struct acvp_modinfo_ctx modinfo;
 	struct acvp_req_ctx req_details;
 	struct acvp_datastore_ctx datastore;
 	struct acvp_opts_ctx options;
+	struct acvp_rename_ctx *rename;
 };
 
 /**
@@ -282,6 +334,26 @@ struct acvp_ctx {
 int acvp_init(const uint8_t *seed, uint32_t seed_len, time_t last_gen,
 	      void (*last_gen_cb)(time_t now));
 
+/**
+ * @brief Load an ACVP Proxy extension
+ *
+ * The ACVP Proxy requires one or more module implementation definitions
+ * (commonly found in lib/module_implementations/). To allow out-of-tree
+ * module definition, this API allows providing a path name to a shared
+ * library that contains the module implementation definition.
+ *
+ * The out-of-tree module implementation definition must use the macro
+ * ACVP_EXTENSION to announce its data structure, i.e. provide the pointer
+ * to the struct def_algo_map as the root of the module definition.
+ *
+ * An example Makefile for an out-of-tree compilation is provided in
+ * helper/Makefile.out-of-tree.
+ *
+ * @param path [in] Path name of the shared library
+ * @return 0 on success, < 0 on error
+ */
+int acvp_load_extension(const char *path);
+
 /**
  * @brief Release ACVP Proxy library
  *
@@ -349,9 +421,9 @@ int acvp_set_net(const char *server_name, unsigned int port, const char *ca,
  *
  * @param ctx [in] ACVP Proxy library context
  * @param caller_search [in] Set the search criteria to find the cipher
- *			    implementation definition.
+ *			     implementation definition.
  * @param specific_ver [in] Provide a version number that shall be registered
- *		      with CAVP instead of @param moduleversion.
+ *			    with CAVP instead of moduleversion.
  * @return 0 on success, < 0 on error
  */
 int acvp_set_module(struct acvp_ctx *ctx,
@@ -434,6 +506,14 @@ int acvp_list_available_ids(const struct acvp_ctx *ctx);
  */
 int acvp_list_verdicts(const struct acvp_ctx *ctx);
 
+/**
+ * @brief List all test session certificate numbers
+ *
+ * @param ctx [in] ACVP Proxy library context
+ * @return 0 on success, < 0 on error
+ */
+int acvp_list_certificates(const struct acvp_ctx *ctx);
+
 /**
  * @brief Refresh JWT authentication token for all already obtained test vectors
  *	 limited by the search criteria.
@@ -455,6 +535,14 @@ int acvp_refresh_authtoken(struct acvp_ctx *ctx, unsigned int submit_vsid);
  */
 int acvp_versionstring(char *buf, size_t buflen);
 
+/**
+ * @brief Return version string of the ACVP Proxy library in a numeric
+ * expression
+ *
+ * @return numeric version
+ */
+uint32_t acvp_versionstring_numeric(void);
+
 /**
  * @brief Dump all registered module definitions to STDERR
  *
@@ -522,22 +610,22 @@ int acvp_set_options(struct acvp_ctx *ctx, const struct acvp_opts_ctx *options);
  * retry information can be disregarded.
  *
  * One function invocation returns one testid for a given index pointer. The
- * caller shall invoke this function repeatedly starting with @param idx_ptr
- * containing a zero. Every subsequent invocation shall use the @param idx_ptr
+ * caller shall invoke this function repeatedly starting with idx_ptr
+ * containing a zero. Every subsequent invocation shall use the idx_ptr
  * value that was returned from the previous round. If the return code -ENOENT
  * is returned, the caller shall stop the iteration. Every recorded testid
- * is returned with @param testid.
+ * is returned with testid.
  *
  * @param idx_ptr [in/out] Index pointer to obtain the testID
  * @param testid [out] TestID at the given index pointer
  *
  * @return 0 on success, -ENOENT identifies that there is no testid for given
- *	   @param idx_ptr, < 0 on other error
+ *	   idx_ptr, < 0 on other error
  */
 int acvp_list_failed_testid(int *idx_ptr, uint32_t *testid);
 
 /**
- * @brief List all vsIDs with a verdict provided in @param passed
+ * @brief List all vsIDs with a verdict provided in passed
  *
  * During the download of the vsIDs, the verdict returned from the ACVP server
  * is processed and the global verdict for the vsID is obtained. The failing
@@ -547,12 +635,12 @@ int acvp_list_failed_testid(int *idx_ptr, uint32_t *testid);
  * verdicts. Naturally, the data is only present after a
  *
  * @param idx_ptr [in/out] Index pointer to obtain the testID
- * @param testid [out] TestID at the given index pointer
+ * @param vsid [out] VsID at the given index pointer
  * @param passed [in] Boolean whether to look for passing verdicts (true) or
  *		      failing verdicts (false).
  *
  * @return 0 on success, -ENOENT identifies that there is no testid for given
- *	   @param idx_ptr, < 0 on other error
+ *	   idx_ptr, < 0 on other error
  */
 int acvp_list_verdict_vsid(int *idx_ptr, uint32_t *vsid, bool passed);
 
@@ -574,6 +662,17 @@ int acvp_cipher_get(const struct acvp_ctx *ctx,
 		    char *ciphername[], size_t ciphername_arraylen,
 		    const char *pathname);
 
+/**
+ * @brief Rename the module information
+ *
+ * This call assumes that ctx->rename_ctx is filled in with the information
+ * to be changed.
+ *
+ * @param ctx [in] ACVP Proxy library context
+ * @return 0 on success, < 0 on error
+ */
+int acvp_rename_module(const struct acvp_ctx *ctx);
+
 #ifdef __cplusplus
 }
 #endif
