@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -40,11 +40,11 @@ extern "C"
 			* zero, the API is not considered stable
 			* and can change without a bump of the
 			* major version). */
-#define MINVERSION 0   /* API compatible, ABI may change,
+#define MINVERSION 2   /* API compatible, ABI may change,
 			* functional enhancements only, consumer
 			* can be left unchanged if enhancements are
 			* not considered. */
-#define PATCHLEVEL 0   /* API / ABI compatible, no functional
+#define PATCHLEVEL 2   /* API / ABI compatible, no functional
 			* changes, no enhancements, bug fixes
 			* only. */
 
@@ -53,40 +53,88 @@ extern "C"
  */
 int acvp_req_set_algo_sym(const struct def_algo_sym *sym,
 			  struct json_object *entry);
+int acvp_req_set_prereq_sym(const struct def_algo_sym *sym,
+			    struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_sha(const struct def_algo_sha *sha,
 			  struct json_object *entry);
 int acvp_req_set_algo_shake(const struct def_algo_shake *shake,
 			    struct json_object *entry);
+
 int acvp_req_set_algo_hmac(const struct def_algo_hmac *hmac,
 			   struct json_object *entry);
+int acvp_req_set_prereq_hmac(const struct def_algo_hmac *hmac,
+			     struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_cmac(const struct def_algo_cmac *cmac,
 			   struct json_object *entry);
+int acvp_req_set_prereq_cmac(const struct def_algo_cmac *cmac,
+			     struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_drbg(const struct def_algo_drbg *drbg,
 			   struct json_object *entry);
+int acvp_req_set_prereq_drbg(const struct def_algo_drbg *drbg,
+			     struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_rsa(const struct def_algo_rsa *rsa,
 			  struct json_object *entry);
+int acvp_req_set_prereq_rsa(const struct def_algo_rsa *rsa,
+			    struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_ecdsa(const struct def_algo_ecdsa *ecdsa,
 			    struct json_object *entry);
+int acvp_req_set_prereq_ecdsa(const struct def_algo_ecdsa *ecdsa,
+			      struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_eddsa(const struct def_algo_eddsa *eddsa,
 			    struct json_object *entry);
+int acvp_req_set_prereq_eddsa(const struct def_algo_eddsa *eddsa,
+			      struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_dsa(const struct def_algo_dsa *dsa,
 			  struct json_object *entry);
+int acvp_req_set_prereq_dsa(const struct def_algo_dsa *dsa,
+			    struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kas_ecc(const struct def_algo_kas_ecc *kas_ecc,
 			      struct json_object *entry);
+int acvp_req_set_prereq_kas_ecc(const struct def_algo_kas_ecc *kas_ecc,
+				struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kas_ffc(const struct def_algo_kas_ffc *kas_ffc,
 			      struct json_object *entry);
+int acvp_req_set_prereq_kas_ffc(const struct def_algo_kas_ffc *kas_ffc,
+			      struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kdf_ssh(const struct def_algo_kdf_ssh *kdf_ssh,
 			      struct json_object *entry);
+int acvp_req_set_prereq_kdf_ssh(const struct def_algo_kdf_ssh *kdf_ssh,
+				struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kdf_ikev1(const struct def_algo_kdf_ikev1 *kdf_ikev1,
 			        struct json_object *entry);
+int acvp_req_set_prereq_kdf_ikev1(const struct def_algo_kdf_ikev1 *kdf_ikev1,
+				  struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kdf_ikev2(const struct def_algo_kdf_ikev2 *kdf_ikev2,
 			        struct json_object *entry);
+int acvp_req_set_prereq_kdf_ikev2(const struct def_algo_kdf_ikev2 *kdf_ikev2,
+				  struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kdf_tls(const struct def_algo_kdf_tls *kdf_tls,
 			      struct json_object *entry);
+int acvp_req_set_prereq_kdf_tls(const struct def_algo_kdf_tls *kdf_tls,
+				struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_kdf_108(const struct def_algo_kdf_108 *kdf_108,
 			      struct json_object *entry);
+int acvp_req_set_prereq_kdf_108(const struct def_algo_kdf_108 *kdf_108,
+				struct json_object *entry, bool publish);
+
 int acvp_req_set_algo_pbkdf(const struct def_algo_pbkdf *pbkdf,
 			    struct json_object *entry);
+int acvp_req_set_prereq_pbkdf(const struct def_algo_pbkdf *pbkdf,
+			      struct json_object *entry, bool publish);
 
 /* Data structure used to exchange information with network backend. */
 struct acvp_na_ex {
@@ -145,18 +193,16 @@ enum acvp_test_verdict {
 	acvp_verdict_fail,
 	acvp_verdict_pass,
 	acvp_verdict_unverified,
+	acvp_verdict_unreceived,
+	acvp_verdict_downloadpending
 };
 
 struct acvp_test_verdict_status {
 	enum acvp_test_verdict verdict;
+	char *cipher_name;
+	char *cipher_mode;
 };
 
-static inline void acvp_store_verdict(struct acvp_test_verdict_status *verdict,
-				      bool test_passed)
-{
-	verdict->verdict = test_passed ? acvp_verdict_pass : acvp_verdict_fail;
-}
-
 /**
  * @brief Data structure instantiated for either request or submission with
  *	  data required for this operation only. The lifetime of an instance
@@ -184,10 +230,10 @@ struct acvp_testid_ctx {
  *	  submission or request. The lifetime of an instance of this data
  *	  structure is limited to one vsID operation only.
  *
- * Note: albeit @param testid_ctx is constified, the parameter server_auth
+ * Note: albeit testid_ctx is constified, the parameter server_auth
  * is allowed to be updated. This is appropriate as the server_auth includes
  * proper locking to serialize write-like changes. This is achieved by
- * explicitly un-constify the @param server_auth.
+ * explicitly un-constify the server_auth.
  */
 struct acvp_vsid_ctx {
 	uint32_t vsid;
@@ -202,9 +248,6 @@ struct acvp_vsid_ctx {
 	 * test response and download the verdict.
 	 */
 
-	/* The caller requested to resubmit the vsID test result */
-	bool resubmit_result;
-
 	/* The vsID test verdict file is present */
 	bool verdict_file_present;
 
@@ -258,8 +301,10 @@ struct acvp_vsid_ctx {
  * @acvp_datastore_write_authoken: Store the authtoken found in datastore->auth.
  * @acvp_datastore_read_authtoken: Read the authtoken from the storage location
  *				   and place it into datastore->auth.
- * @acvp_datastore_get_testid_verdict: Get verdict information for testID
- * @acvp_datastore_get_vsid_verdict: Get verdict information for vsID
+ * @acvp_datastore_get_testid_verdict Get verdict information for testID
+ * @acvp_datastore_get_vsid_verdict Get verdict information for vsID
+ * @acvp_datastore_file_rename_version Rename module: change version number
+ * @acvp_datastore_file_rename_name Rename module: change module name
  */
 struct acvp_datastore_be {
 	int (*acvp_datastore_find_testsession)(
@@ -289,6 +334,10 @@ struct acvp_datastore_be {
 	int (*acvp_datastore_get_testid_verdict)(
 		struct acvp_testid_ctx *testid_ctx);
 	int (*acvp_datastore_get_vsid_verdict)(struct acvp_vsid_ctx *vsid_ctx);
+	int (*acvp_datastore_rename_version)
+		(const struct acvp_testid_ctx *testid_ctx, char *newversion);
+	int (*acvp_datastore_rename_name)
+		(const struct acvp_testid_ctx *testid_ctx, char *newname);
 };
 
 /**
@@ -415,9 +464,9 @@ int acvp_process_retry_testid(const struct acvp_testid_ctx *testid_ctx,
  * Some resources require paging in order to avoid returning large
  * amounts of data.
  *
- * This function performs the HTTP GET operation and invokes @param cb
+ * This function performs the HTTP GET operation and invokes cb
  * for each found data entry. Note, the function returns < 0 on error if
- * the @param cb returns < 0. If @param cb returns EINTR (positive value),
+ * the cb returns < 0. If cb returns EINTR (positive value),
  * the loop terminates. If the callback return 0, the loop iteration continues.
  *
  * @param testid_ctx TestID context with set credentials
@@ -471,7 +520,8 @@ void acvp_op_enable(void);
  ************************************************************************/
 
 enum acvp_http_type {
-	acvp_http_post = 1,
+	acvp_http_none = 0,
+	acvp_http_post,
 	acvp_http_put,
 	acvp_http_delete,
 	acvp_http_get
@@ -507,6 +557,19 @@ int acvp_meta_register(const struct acvp_testid_ctx *testid_ctx,
  */
 int acvp_get_id_from_url(const char *url, uint32_t *id);
 
+/**
+ * @brief Convert search return code to HTTP request type
+ *
+ * @param search_errno [in] Error number of the search operation
+ * @param type [in] request type
+ * @param ctx_opts [in] Options of invocation
+ * @param id [in] ID field of search operation
+ * @param http_type [out] Returned HTTP type.
+ */
+int acvp_search_to_http_type(int search_errno, unsigned int type,
+			     const struct acvp_opts_ctx *ctx_opts, uint32_t id,
+			     enum acvp_http_type *http_type);
+
 /**
  * @brief Match two strings
  */
@@ -516,7 +579,13 @@ int acvp_str_match(const char *exp, const char *found, uint32_t id);
  * @brief Obtain the verdict from the JSON data.
  */
 int acvp_get_verdict_json(const struct acvp_buf *verdict_buf,
-			  bool *test_passed);
+			  enum acvp_test_verdict *verdict_stat);
+
+/*
+ * @brief Obtain cipher information from JSON data.
+ */
+int acvp_get_algoinfo_json(const struct acvp_buf *buf,
+			   struct acvp_test_verdict_status *verdict);
 
 /**
  * @brief Helper to perform HTTP operation
@@ -665,11 +734,11 @@ int acvp_store_module_debug(const struct acvp_testid_ctx *testid_ctx,
 int acvp_store_person_debug(const struct acvp_testid_ctx *testid_ctx,
 			    const struct acvp_buf *buf, int err);
 int acvp_store_file(const struct acvp_testid_ctx *testid_ctx,
-			    const struct acvp_buf *buf, int err,
-			    const char *file);
+		    const struct acvp_buf *buf, int err,
+		    const char *file);
 
-int acvp_req_check_string(char *string, unsigned int slen);
-int acvp_req_check_filename(char *string, unsigned int slen);
+int acvp_req_check_string(char *string, size_t slen);
+int acvp_req_check_filename(char *string, size_t slen);
 
 bool acvp_req_is_production(void);
 
@@ -693,6 +762,8 @@ bool acvp_req_is_production(void);
 #define ACVP_DS_MESSAGESIZECONSTRAINT		"messagesizeconstraint.txt"
 /* Approval / Certificate ID */
 #define ACVP_DS_TESTSESSIONCERTIFICATEID	"testsession_certificate_id.txt"
+/* Certificate details */
+#define ACVP_DS_TESTSESSIONCERTIFICATEINFO	"testsession_certificate_info.json"
 /* File that will hold the test verdict from the ACVP server */
 #define ACVP_DS_VERDICT				"verdict.json"
 /* File that contains the time stamp when the vector was uploaded */
@@ -708,7 +779,7 @@ bool acvp_req_is_production(void);
 #define ACVP_DS_UPLOADDURATION			"upload_duration.txt"
 /* File containing the version information of the data store */
 #define ACVP_DS_VERSIONFILE			"datastore_version.txt"
-#define ACVP_DS_VERSION				2
+#define ACVP_DS_VERSION				3
 /* File holding the unambiguous search criteria to look up cipher definition */
 #define ACVP_DS_DEF_REFERENCE			"definition_reference.json"
 /* File holding the ACVP request */
